/* 
  ArduinoOcfe

  Demonstrates the capabilities and ease of use of the Ocfe protocol on Arduino.

  This example code is in the public domain.


  Pre-requisites : 
  - the OCFE folder containing protocol files must be copied into the Arduino IDE 
    libraries folder (typically "C:\Program Files (x86)\Arduino\libraries")

  More information at : www.objectis-software.com
*/

#include "OcfEmbedded.h"

int AnalogIn[8];
int DigitalIn[14];
int DigitalOut[14];
int BlinkPeriod[14];

OCFE_BEGIN(ocfEntries)
  OCFE_VARIABLE_READONLY(AnalogIn[0], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(AnalogIn[1], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(AnalogIn[2], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(AnalogIn[3], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(AnalogIn[4], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(AnalogIn[5], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(AnalogIn[6], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(AnalogIn[7], OCFE_TYPE_INT)
  OCFE_VARIABLE_READONLY(DigitalIn[2], OCFE_TYPE_BOOL)
  OCFE_VARIABLE_READONLY(DigitalIn[3], OCFE_TYPE_BOOL)
  OCFE_VARIABLE_READONLY(DigitalIn[4], OCFE_TYPE_BOOL)
  OCFE_VARIABLE_READONLY(DigitalIn[5], OCFE_TYPE_BOOL)
  OCFE_VARIABLE_READONLY(DigitalIn[6], OCFE_TYPE_BOOL)
  OCFE_VARIABLE_READONLY(DigitalIn[7], OCFE_TYPE_BOOL)
  OCFE_VARIABLE_READONLY(DigitalIn[8], OCFE_TYPE_BOOL)
  OCFE_VARIABLE_READONLY(DigitalIn[9], OCFE_TYPE_BOOL)
  OCFE_VARIABLE(DigitalOut[10], OCFE_TYPE_BOOL)
  OCFE_VARIABLE(DigitalOut[11], OCFE_TYPE_BOOL)
  OCFE_VARIABLE(DigitalOut[12], OCFE_TYPE_BOOL)
  OCFE_VARIABLE(DigitalOut[13], OCFE_TYPE_BOOL)
  OCFE_VARIABLE(BlinkPeriod[10], OCFE_TYPE_INT)
  OCFE_VARIABLE(BlinkPeriod[11], OCFE_TYPE_INT)
  OCFE_VARIABLE(BlinkPeriod[12], OCFE_TYPE_INT)
  OCFE_VARIABLE(BlinkPeriod[13], OCFE_TYPE_INT)
OCFE_END;

unsigned long lastBlinkTime[14];

void setup()
{
  Serial.begin(230400);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }
  pinMode(2, INPUT);
  pinMode(3, INPUT);
  pinMode(4, INPUT);
  pinMode(5, INPUT);
  pinMode(6, INPUT);
  pinMode(7, INPUT);
  pinMode(8, INPUT);
  pinMode(9, INPUT);
  pinMode(10, OUTPUT);
  pinMode(11, OUTPUT);
  pinMode(12, OUTPUT);
  pinMode(13, OUTPUT);
  for (int i = 10; i <= 13; i++)
  {
    lastBlinkTime[i] = millis();
    BlinkPeriod[i] = 0;
  }
  BlinkPeriod[13] = 1000;    
}

OcfeStreamContext streamContext;
int updateState = 0; // 0..7 -> Update analog, 8-> Update digital

void loop()
{
  // Processing of the serial communication protocol with OcfE
  char c;
  if (Serial.available() > 0) 
  {
    c = Serial.read();
    OcfeCharacterReceived(ocfEntries, &streamContext, c); 
  }
  if (OcfeCharacterToSend(&streamContext, &c))
    Serial.write(c);
    
  // Update
  if (updateState <= 7)
  {
    AnalogIn[updateState] = analogRead(updateState);
    updateState++;
  }
  else
  {
    for (int i = 2; i <= 9; i++)
      DigitalIn[i] = digitalRead(i);
    for (int i = 10; i <= 13; i++)
    {
      if ((BlinkPeriod[i] != 0) && (millis() - lastBlinkTime[i] >= BlinkPeriod[i] / 2))
      {      
        lastBlinkTime[i] = millis();
        DigitalOut[i] = !DigitalOut[i];
      }
      digitalWrite(i, DigitalOut[i]);
    }
    updateState = 0;
  }
}


