/////////////////////////////////////////////////////////////////////////////////
//                                                                             //
// <copyright company="Objectis">                                              //
//                                                                             //
// Copyright (c) 2013, All Right Reserved, http://www.objectis.ch/             //
//                                                                             //
// This source is subject to the Objectis Permissive License.                  //
// Please see the License contract for more information.                       //      
// All other rights reserved.                                                  //
//                                                                             //
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY      //
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE         //
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A                  //
// PARTICULAR PURPOSE.                                                         //
//                                                                             //
// </copyright>                                                                //
//                                                                             //
/////////////////////////////////////////////////////////////////////////////////

#ifndef __OCFE_PROTOCOL_H_
#define __OCFE_PROTOCOL_H_

#include "OcfEmbeddedPort.h"


typedef enum
{
	OCFE_VARIABLE_OPTION_NONE = 1,
	OCFE_VARIABLE_OPTION_READONLY = 2

#ifdef OCFE_ENABLE_FUNCTION
	,OCFE_VARIABLE_OPTION_FUNCTION = 4
#endif
} OcfeVariableOptions;

typedef struct
{
	const char* name;
	void * address;
	unsigned char type;
	unsigned char options;
#ifdef OCFE_ENABLE_STRING
	unsigned char size;
	unsigned char __reserved__;
#endif
} OcfeVariableEntry;

#define OCFE_BEGIN(ocfeDictionaryName) \
	OcfeVariableEntry ocfeDictionaryName[] = \
{

#ifdef OCFE_ENABLE_STRING
#define OCFE_VARIABLE_CUSTOM(variable, ocfType, options)	\
{ (const char*)#variable, &variable, ocfType, options, sizeof variable, 0 },

#define OCFE_END \
{0, 0, 0, 0, 0, 0}};

#define OCFE_FUNCTION(function, type) \
{#function, (void*)function, type, OCFE_VARIABLE_OPTION_FUNCTION, 0, 0},

#define OCFE_FUNCTION_VOID(function) \
{#function, (void*)function, OCFE_TYPE_FUNCTION_VOID, OCFE_VARIABLE_OPTION_FUNCTION, 0, 0},

#define OCFE_FUNCTION_RET(function) \
{#function, (void*)function, OCFE_TYPE_FUNCTION_RET, OCFE_VARIABLE_OPTION_FUNCTION, 0, 0},

#else
#define OCFE_VARIABLE_CUSTOM(variable, ocfType, options) \
{ #variable, &variable, ocfType, options},

#define OCFE_END \
{0,0,0,0}};

#define OCFE_FUNCTION(function, type) \
{#function, (void*)function, type, OCFE_VARIABLE_OPTION_FUNCTION},

#define OCFE_FUNCTION_VOID(function) \
{#function, (void*)function, OCFE_TYPE_FUNCTION_VOID, OCFE_VARIABLE_OPTION_FUNCTION},

#define OCFE_FUNCTION_RET(function) \
{#function, (void*)function, OCFE_TYPE_FUNCTION_RET, OCFE_VARIABLE_OPTION_FUNCTION},
#endif

#define OCFE_VARIABLE(variable, ocfType)	\
	OCFE_VARIABLE_CUSTOM(variable, ocfType, OCFE_VARIABLE_OPTION_NONE)

#define OCFE_VARIABLE_READONLY(variable, ocfType)	\
	OCFE_VARIABLE_CUSTOM(variable, ocfType, OCFE_VARIABLE_OPTION_READONLY)


// Frame structures :
// Get a metadata        : <FrameNumber>|G|<MetaDataIndice>|<Fletcher16CheckSum>
// Return a metadata     : <FrameNumber>|M|<VariableName>|<VariableOptions>|<Fletcher16CheckSum>
// Read a variable       : <FrameNumber>|R|<VariableName>|<Fletcher16CheckSum>
// Return a variable     : <FrameNumber>|V|Value|<Fletcher16CheckSum>
// Write a variable      : <FrameNumber>|W|<VariableName>|Value|<Fletcher16CheckSum>
// Acknowledge a write   : <FrameNumber>|A|<Fletcher16CheckSum>
// Data error            : <FrameNumber>|E|<Message>|<Fletcher16CheckSum>
// Invalid frame/command : INVALID
// where | denotes Ocfe Separator

#define OCFE_FRAME_TYPE_REQ_METADATA 'G'
#define OCFE_FRAME_TYPE_IND_METADATA 'M'
#define OCFE_FRAME_TYPE_REQ_VARIABLE 'R'
#define OCFE_FRAME_TYPE_IND_VARIABLE 'V'
#define OCFE_FRAME_TYPE_REQ_WRITE 'W'
#define OCFE_FRAME_TYPE_IND_WRITE 'A'
#define OCFE_FRAME_TYPE_IND_ERROR 'E'

#define OCFE_FRAME_TYPE_IND_INVALID "INVALID\n"
#define OCFE_FRAME_TYPE_IND_INVALID_SIZE 8

#define OCFE_FRAME_CHECKSUM_ERROR_MSG "CHECKSUM_ERROR\n"
#define OCFE_FRAME_CHECKSUM_ERROR_SIZE 15

#define OCFE_FRAME_INDEX_OUT_RANGE_MSG	"Index out of range"
#define OCFE_FRAME_INDEX_OUT_RANGE_SIZE	18

#define OCFE_FRAME_ERROR_STRING_SIZE_OVERFLOW_MSG	"String size overflow"

#define OCFE_COMMAND_SEPARATOR ((char)0)
#define OCFE_COMMAND_TERMINATOR ((char)10)

#define OCFE_COMMAND_LENGTH 1
#define OCFE_SEPARATORS_LENGTH 6

#define OCFE_MAX_FRAME_NUMERATOR_SIZE	2

#ifdef OCFE_ENABLE_CHECKSUM
#define OCFE_CHECKSUM_LENGTH 5
#else
#define OCFE_CHECKSUM_LENGTH 0
#endif

#ifndef OCFE_ENABLE_FRAME_NUMERATOR
#undef OCFE_MAX_FRAME_NUMERATOR_SIZE
#define OCFE_MAX_FRAME_NUMERATOR_SIZE 0
#endif

#define OCFE_COMMUNICATION_BUFFER_SIZE (OCFE_COMMAND_LENGTH + OCFE_CHECKSUM_LENGTH + OCFE_MAX_FRAME_NUMERATOR_SIZE + OCFE_MAX_VAR_NAME_LENGTH + OCFE_MAX_VALUE_LENGTH + OCFE_SEPARATORS_LENGTH)


#define OCFE_MAX_FRAME_ELEMENT_COUNT 6

#endif
